// Copyright (C) 2025 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only

#ifndef TST_GENERATED_LSP_TYPES_H
#define TST_GENERATED_LSP_TYPES_H

#include <QtCore/qobject.h>
#include <QtLanguageServer/private/qlanguageserverspectypes_p.h>

class tst_generated_lsp_types : public QObject
{
    Q_OBJECT

private slots:
    void tst_types();
};

#endif // TST_GENERATED_LSP_TYPES_H
